# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" This package contains functions common for distribution management.  TODO: The following has been copied from a very old version of draft Part 11, so the references are wrong, but we store the knowledge here to reuse later: 'Locations are logical entities which are related to a geographical position. Locations can be defined as points, lines or polygons. Location serves as a parent class for e.g. Zone, WorkLocation or ServiceLocation. Both Assets and PowerSystemResources are typically associated to a location. Aside from coordinates, useful properties of Locations can include Directions (i.e. driving instructions) and relationships to Organizations. ActivityRecord is a generalized class used to track the history of an object (e.g. Asset, PowerSystemResource, Customer, Location, Organisation or ErpContact). An ActivityRecord is a type of Document. Key properties of an ActivityRecord include statusDate, status, statusReason and remarks. TODO: Update attribute names. The graphical and geographical aspects of Assets, Locations and PowerSystemResources are managed using Graphical Markup Language (GML), which was defined by the Open GIS Consortium.  Using GML, a diagram is a collection of presentation objects. This package defines the classes Diagram and Presentation. TODO: These are now under Common package.'
"""

from cim.iec61970.core import IdentifiedObject
from cim.iec61968.common import Document
from cim import Element

# <<< imports
# @generated
# >>> imports

ns_prefix = "cim.infcommon"

ns_uri = "http://iec.ch/TC57/2009/CIM-schema-cim14#InfCommon"

class Role(IdentifiedObject):
    """ Enumeration of potential roles that might be played by one object relative to another.
    """
    # <<< role
    # @generated
    def __init__(self, category='', status=None, **kw_args):
        """ Initialises a new 'Role' instance.
        """
        # Category of role. 
        self.category = category


        self.status = status


        super(Role, self).__init__(**kw_args)
    # >>> role

    # <<< status
    # @generated
    status = None
    # >>> status


    def __str__(self):
        """ Returns a string representation of the Role.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< role.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the Role.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "Role", self.uri)
        if format:
            indent += ' ' * depth

        if self.status is not None:
            s += '%s<%s:Role.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        s += '%s<%s:Role.category>%s</%s:Role.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "Role")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> role.serialize


class ScheduledEvent(IdentifiedObject):
    """ Signifies an event to trigger one or more activities, such as reading a meter, recalculating a bill, requesting work, when generating units must be scheduled for maintenance, when a transformer is scheduled to be refurbished, etc.
    """
    # <<< scheduled_event
    # @generated
    def __init__(self, duration=0.0, category='', document=None, assets=None, activity_record=None, schedule_parameter_info=None, status=None, time_point=None, **kw_args):
        """ Initialises a new 'ScheduledEvent' instance.
        """
        # Duration of the scheduled event, for example, the time to ramp between values. 
        self.duration = duration

        # Category of scheduled event. 
        self.category = category


        self._document = None
        self.document = document

        self._assets = []
        if assets is not None:
            self.assets = assets
        else:
            self.assets = []

        self._activity_record = None
        self.activity_record = activity_record

        self._schedule_parameter_info = None
        self.schedule_parameter_info = schedule_parameter_info

        self.status = status

        self._time_point = None
        self.time_point = time_point


        super(ScheduledEvent, self).__init__(**kw_args)
    # >>> scheduled_event

    # <<< document
    # @generated
    def get_document(self):
        """ 
        """
        return self._document

    def set_document(self, value):
        if self._document is not None:
            filtered = [x for x in self.document.scheduled_events if x != self]
            self._document._scheduled_events = filtered

        self._document = value
        if self._document is not None:
            self._document._scheduled_events.append(self)

    document = property(get_document, set_document)
    # >>> document

    # <<< assets
    # @generated
    def get_assets(self):
        """ 
        """
        return self._assets

    def set_assets(self, value):
        for p in self._assets:
            filtered = [q for q in p.scheduled_events if q != self]
            self._assets._scheduled_events = filtered
        for r in value:
            if self not in r._scheduled_events:
                r._scheduled_events.append(self)
        self._assets = value

    assets = property(get_assets, set_assets)

    def add_assets(self, *assets):
        for obj in assets:
            if self not in obj._scheduled_events:
                obj._scheduled_events.append(self)
            self._assets.append(obj)

    def remove_assets(self, *assets):
        for obj in assets:
            if self in obj._scheduled_events:
                obj._scheduled_events.remove(self)
            self._assets.remove(obj)
    # >>> assets

    # <<< activity_record
    # @generated
    def get_activity_record(self):
        """ 
        """
        return self._activity_record

    def set_activity_record(self, value):
        if self._activity_record is not None:
            self._activity_record._scheduled_event = None

        self._activity_record = value
        if self._activity_record is not None:
            self._activity_record._scheduled_event = self

    activity_record = property(get_activity_record, set_activity_record)
    # >>> activity_record

    # <<< schedule_parameter_info
    # @generated
    def get_schedule_parameter_info(self):
        """ 
        """
        return self._schedule_parameter_info

    def set_schedule_parameter_info(self, value):
        if self._schedule_parameter_info is not None:
            filtered = [x for x in self.schedule_parameter_info.scheduled_events if x != self]
            self._schedule_parameter_info._scheduled_events = filtered

        self._schedule_parameter_info = value
        if self._schedule_parameter_info is not None:
            self._schedule_parameter_info._scheduled_events.append(self)

    schedule_parameter_info = property(get_schedule_parameter_info, set_schedule_parameter_info)
    # >>> schedule_parameter_info

    # <<< status
    # @generated
    status = None
    # >>> status

    # <<< time_point
    # @generated
    def get_time_point(self):
        """ 
        """
        return self._time_point

    def set_time_point(self, value):
        if self._time_point is not None:
            filtered = [x for x in self.time_point.scheduled_events if x != self]
            self._time_point._scheduled_events = filtered

        self._time_point = value
        if self._time_point is not None:
            self._time_point._scheduled_events.append(self)

    time_point = property(get_time_point, set_time_point)
    # >>> time_point


    def __str__(self):
        """ Returns a string representation of the ScheduledEvent.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< scheduled_event.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the ScheduledEvent.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "ScheduledEvent", self.uri)
        if format:
            indent += ' ' * depth

        if self.document is not None:
            s += '%s<%s:ScheduledEvent.document rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.document.uri)
        for obj in self.assets:
            s += '%s<%s:ScheduledEvent.assets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.activity_record is not None:
            s += '%s<%s:ScheduledEvent.activity_record rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.activity_record.uri)
        if self.schedule_parameter_info is not None:
            s += '%s<%s:ScheduledEvent.schedule_parameter_info rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.schedule_parameter_info.uri)
        if self.status is not None:
            s += '%s<%s:ScheduledEvent.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        if self.time_point is not None:
            s += '%s<%s:ScheduledEvent.time_point rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.time_point.uri)
        s += '%s<%s:ScheduledEvent.duration>%s</%s:ScheduledEvent.duration>' % \
            (indent, ns_prefix, self.duration, ns_prefix)
        s += '%s<%s:ScheduledEvent.category>%s</%s:ScheduledEvent.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "ScheduledEvent")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> scheduled_event.serialize


class Skill(Document):
    """ Proficiency level of a craft, which is required to operate or maintain a particular type of asset and/or perform certain types of work.
    """
    # <<< skill
    # @generated
    def __init__(self, level='master', effective_date_time='', crafts=None, qualification_requirements=None, erp_person=None, certification_period=None, **kw_args):
        """ Initialises a new 'Skill' instance.
        """
        # Level of skill for a Craft. Values are: "master", "other", "standard", "apprentice"
        self.level = 'master'

        # Date and time the skill became effective. 
        self.effective_date_time = effective_date_time


        self._crafts = []
        if crafts is not None:
            self.crafts = crafts
        else:
            self.crafts = []

        self._qualification_requirements = []
        if qualification_requirements is not None:
            self.qualification_requirements = qualification_requirements
        else:
            self.qualification_requirements = []

        self._erp_person = None
        self.erp_person = erp_person

        self.certification_period = certification_period


        super(Skill, self).__init__(**kw_args)
    # >>> skill

    # <<< crafts
    # @generated
    def get_crafts(self):
        """ 
        """
        return self._crafts

    def set_crafts(self, value):
        for p in self._crafts:
            filtered = [q for q in p.skills if q != self]
            self._crafts._skills = filtered
        for r in value:
            if self not in r._skills:
                r._skills.append(self)
        self._crafts = value

    crafts = property(get_crafts, set_crafts)

    def add_crafts(self, *crafts):
        for obj in crafts:
            if self not in obj._skills:
                obj._skills.append(self)
            self._crafts.append(obj)

    def remove_crafts(self, *crafts):
        for obj in crafts:
            if self in obj._skills:
                obj._skills.remove(self)
            self._crafts.remove(obj)
    # >>> crafts

    # <<< qualification_requirements
    # @generated
    def get_qualification_requirements(self):
        """ 
        """
        return self._qualification_requirements

    def set_qualification_requirements(self, value):
        for p in self._qualification_requirements:
            filtered = [q for q in p.skills if q != self]
            self._qualification_requirements._skills = filtered
        for r in value:
            if self not in r._skills:
                r._skills.append(self)
        self._qualification_requirements = value

    qualification_requirements = property(get_qualification_requirements, set_qualification_requirements)

    def add_qualification_requirements(self, *qualification_requirements):
        for obj in qualification_requirements:
            if self not in obj._skills:
                obj._skills.append(self)
            self._qualification_requirements.append(obj)

    def remove_qualification_requirements(self, *qualification_requirements):
        for obj in qualification_requirements:
            if self in obj._skills:
                obj._skills.remove(self)
            self._qualification_requirements.remove(obj)
    # >>> qualification_requirements

    # <<< erp_person
    # @generated
    def get_erp_person(self):
        """ 
        """
        return self._erp_person

    def set_erp_person(self, value):
        if self._erp_person is not None:
            filtered = [x for x in self.erp_person.skills if x != self]
            self._erp_person._skills = filtered

        self._erp_person = value
        if self._erp_person is not None:
            self._erp_person._skills.append(self)

    erp_person = property(get_erp_person, set_erp_person)
    # >>> erp_person

    # <<< certification_period
    # @generated
    # Interval between the certification and its expiry.
    certification_period = None
    # >>> certification_period


    def __str__(self):
        """ Returns a string representation of the Skill.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< skill.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the Skill.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "Skill", self.uri)
        if format:
            indent += ' ' * depth

        for obj in self.crafts:
            s += '%s<%s:Skill.crafts rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.qualification_requirements:
            s += '%s<%s:Skill.qualification_requirements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.erp_person is not None:
            s += '%s<%s:Skill.erp_person rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.erp_person.uri)
        if self.certification_period is not None:
            s += '%s<%s:Skill.certification_period rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.certification_period.uri)
        s += '%s<%s:Skill.level>%s</%s:Skill.level>' % \
            (indent, ns_prefix, self.level, ns_prefix)
        s += '%s<%s:Skill.effective_date_time>%s</%s:Skill.effective_date_time>' % \
            (indent, ns_prefix, self.effective_date_time, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        for obj in self.activity_records:
            s += '%s<%s:Document.activity_records rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_organisation_roles:
            s += '%s<%s:Document.erp_organisation_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.scheduled_events:
            s += '%s<%s:Document.scheduled_events rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.from_document_roles:
            s += '%s<%s:Document.from_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.location_roles:
            s += '%s<%s:Document.location_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.power_system_resource_roles:
            s += '%s<%s:Document.power_system_resource_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.network_data_sets:
            s += '%s<%s:Document.network_data_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_person_roles:
            s += '%s<%s:Document.erp_person_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_items:
            s += '%s<%s:Document.change_items rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.measurements:
            s += '%s<%s:Document.measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.doc_status is not None:
            s += '%s<%s:Document.doc_status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.doc_status.uri)
        for obj in self.schedule_parameter_infos:
            s += '%s<%s:Document.schedule_parameter_infos rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.electronic_address is not None:
            s += '%s<%s:Document.electronic_address rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.electronic_address.uri)
        for obj in self.to_document_roles:
            s += '%s<%s:Document.to_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.status is not None:
            s += '%s<%s:Document.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        for obj in self.asset_roles:
            s += '%s<%s:Document.asset_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_sets:
            s += '%s<%s:Document.change_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:Document.subject>%s</%s:Document.subject>' % \
            (indent, ns_prefix, self.subject, ns_prefix)
        s += '%s<%s:Document.revision_number>%s</%s:Document.revision_number>' % \
            (indent, ns_prefix, self.revision_number, ns_prefix)
        s += '%s<%s:Document.category>%s</%s:Document.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        s += '%s<%s:Document.last_modified_date_time>%s</%s:Document.last_modified_date_time>' % \
            (indent, ns_prefix, self.last_modified_date_time, ns_prefix)
        s += '%s<%s:Document.title>%s</%s:Document.title>' % \
            (indent, ns_prefix, self.title, ns_prefix)
        s += '%s<%s:Document.created_date_time>%s</%s:Document.created_date_time>' % \
            (indent, ns_prefix, self.created_date_time, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "Skill")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> skill.serialize


class BankAccount(Document):
    """ Bank account.
    """
    # <<< bank_account
    # @generated
    def __init__(self, account_number='', service_supplier=None, bank=None, bank_statements=None, **kw_args):
        """ Initialises a new 'BankAccount' instance.
        """
        # Account reference number. 
        self.account_number = account_number


        self._service_supplier = None
        self.service_supplier = service_supplier

        self._bank = None
        self.bank = bank

        self._bank_statements = []
        if bank_statements is not None:
            self.bank_statements = bank_statements
        else:
            self.bank_statements = []


        super(BankAccount, self).__init__(**kw_args)
    # >>> bank_account

    # <<< service_supplier
    # @generated
    def get_service_supplier(self):
        """ ServiceSupplier that is owner of this BankAccount.
        """
        return self._service_supplier

    def set_service_supplier(self, value):
        if self._service_supplier is not None:
            filtered = [x for x in self.service_supplier.bank_accounts if x != self]
            self._service_supplier._bank_accounts = filtered

        self._service_supplier = value
        if self._service_supplier is not None:
            self._service_supplier._bank_accounts.append(self)

    service_supplier = property(get_service_supplier, set_service_supplier)
    # >>> service_supplier

    # <<< bank
    # @generated
    def get_bank(self):
        """ Bank that provides this BankAccount.
        """
        return self._bank

    def set_bank(self, value):
        if self._bank is not None:
            filtered = [x for x in self.bank.bank_accounts if x != self]
            self._bank._bank_accounts = filtered

        self._bank = value
        if self._bank is not None:
            self._bank._bank_accounts.append(self)

    bank = property(get_bank, set_bank)
    # >>> bank

    # <<< bank_statements
    # @generated
    def get_bank_statements(self):
        """ All bank statements generated from this bank account.
        """
        return self._bank_statements

    def set_bank_statements(self, value):
        for x in self._bank_statements:
            x._bank_account = None
        for y in value:
            y._bank_account = self
        self._bank_statements = value

    bank_statements = property(get_bank_statements, set_bank_statements)

    def add_bank_statements(self, *bank_statements):
        for obj in bank_statements:
            obj._bank_account = self
            self._bank_statements.append(obj)

    def remove_bank_statements(self, *bank_statements):
        for obj in bank_statements:
            obj._bank_account = None
            self._bank_statements.remove(obj)
    # >>> bank_statements


    def __str__(self):
        """ Returns a string representation of the BankAccount.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< bank_account.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the BankAccount.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "BankAccount", self.uri)
        if format:
            indent += ' ' * depth

        if self.service_supplier is not None:
            s += '%s<%s:BankAccount.service_supplier rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.service_supplier.uri)
        if self.bank is not None:
            s += '%s<%s:BankAccount.bank rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.bank.uri)
        for obj in self.bank_statements:
            s += '%s<%s:BankAccount.bank_statements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:BankAccount.account_number>%s</%s:BankAccount.account_number>' % \
            (indent, ns_prefix, self.account_number, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        for obj in self.activity_records:
            s += '%s<%s:Document.activity_records rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_organisation_roles:
            s += '%s<%s:Document.erp_organisation_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.scheduled_events:
            s += '%s<%s:Document.scheduled_events rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.from_document_roles:
            s += '%s<%s:Document.from_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.location_roles:
            s += '%s<%s:Document.location_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.power_system_resource_roles:
            s += '%s<%s:Document.power_system_resource_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.network_data_sets:
            s += '%s<%s:Document.network_data_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_person_roles:
            s += '%s<%s:Document.erp_person_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_items:
            s += '%s<%s:Document.change_items rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.measurements:
            s += '%s<%s:Document.measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.doc_status is not None:
            s += '%s<%s:Document.doc_status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.doc_status.uri)
        for obj in self.schedule_parameter_infos:
            s += '%s<%s:Document.schedule_parameter_infos rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.electronic_address is not None:
            s += '%s<%s:Document.electronic_address rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.electronic_address.uri)
        for obj in self.to_document_roles:
            s += '%s<%s:Document.to_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.status is not None:
            s += '%s<%s:Document.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        for obj in self.asset_roles:
            s += '%s<%s:Document.asset_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_sets:
            s += '%s<%s:Document.change_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:Document.subject>%s</%s:Document.subject>' % \
            (indent, ns_prefix, self.subject, ns_prefix)
        s += '%s<%s:Document.revision_number>%s</%s:Document.revision_number>' % \
            (indent, ns_prefix, self.revision_number, ns_prefix)
        s += '%s<%s:Document.category>%s</%s:Document.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        s += '%s<%s:Document.last_modified_date_time>%s</%s:Document.last_modified_date_time>' % \
            (indent, ns_prefix, self.last_modified_date_time, ns_prefix)
        s += '%s<%s:Document.title>%s</%s:Document.title>' % \
            (indent, ns_prefix, self.title, ns_prefix)
        s += '%s<%s:Document.created_date_time>%s</%s:Document.created_date_time>' % \
            (indent, ns_prefix, self.created_date_time, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "BankAccount")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> bank_account.serialize


class MarketRole(IdentifiedObject):
    """ Role an organisation plays in a market. Examples include one or more of values defined in MarketRoleKind.
    """
    # <<< market_role
    # @generated
    def __init__(self, kind='other', status=None, organisations=None, **kw_args):
        """ Initialises a new 'MarketRole' instance.
        """
        # Kind of role an organisation plays in a market. Values are: "other", "transmission_service_provider", "planning_authority", "reliability_authority", "transmission_owner", "transmission_planner", "generator_operator", "energy_service_consumer", "generator_owner", "transmission_operator", "compliance_monitor", "distribution_provider", "load_serving_entity", "interchange_authority", "purchasing_selling_entity", "resource_planner", "balancing_authority", "competitive_retailer", "standards_developer"
        self.kind = 'other'


        self.status = status

        self._organisations = []
        if organisations is not None:
            self.organisations = organisations
        else:
            self.organisations = []


        super(MarketRole, self).__init__(**kw_args)
    # >>> market_role

    # <<< status
    # @generated
    status = None
    # >>> status

    # <<< organisations
    # @generated
    def get_organisations(self):
        """ 
        """
        return self._organisations

    def set_organisations(self, value):
        for p in self._organisations:
            filtered = [q for q in p.market_roles if q != self]
            self._organisations._market_roles = filtered
        for r in value:
            if self not in r._market_roles:
                r._market_roles.append(self)
        self._organisations = value

    organisations = property(get_organisations, set_organisations)

    def add_organisations(self, *organisations):
        for obj in organisations:
            if self not in obj._market_roles:
                obj._market_roles.append(self)
            self._organisations.append(obj)

    def remove_organisations(self, *organisations):
        for obj in organisations:
            if self in obj._market_roles:
                obj._market_roles.remove(self)
            self._organisations.remove(obj)
    # >>> organisations


    def __str__(self):
        """ Returns a string representation of the MarketRole.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< market_role.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the MarketRole.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "MarketRole", self.uri)
        if format:
            indent += ' ' * depth

        if self.status is not None:
            s += '%s<%s:MarketRole.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        for obj in self.organisations:
            s += '%s<%s:MarketRole.organisations rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:MarketRole.kind>%s</%s:MarketRole.kind>' % \
            (indent, ns_prefix, self.kind, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "MarketRole")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> market_role.serialize


class Diagram(Document):
    """ GML and/or other means are used for rendering objects on various types of displays(geographic, schematic, other) and maps associated with various coordinate systems.
    """
    # <<< diagram
    # @generated
    def __init__(self, kind='geographic', gml_diagram_objects=None, design_locations=None, gml_coordinate_system=None, **kw_args):
        """ Initialises a new 'Diagram' instance.
        """
        # Kind of this diagram. Values are: "geographic", "schematic", "design_sketch", "internal_view", "other"
        self.kind = 'geographic'


        self._gml_diagram_objects = []
        if gml_diagram_objects is not None:
            self.gml_diagram_objects = gml_diagram_objects
        else:
            self.gml_diagram_objects = []

        self._design_locations = []
        if design_locations is not None:
            self.design_locations = design_locations
        else:
            self.design_locations = []

        self._gml_coordinate_system = None
        self.gml_coordinate_system = gml_coordinate_system


        super(Diagram, self).__init__(**kw_args)
    # >>> diagram

    # <<< gml_diagram_objects
    # @generated
    def get_gml_diagram_objects(self):
        """ 
        """
        return self._gml_diagram_objects

    def set_gml_diagram_objects(self, value):
        for p in self._gml_diagram_objects:
            filtered = [q for q in p.diagrams if q != self]
            self._gml_diagram_objects._diagrams = filtered
        for r in value:
            if self not in r._diagrams:
                r._diagrams.append(self)
        self._gml_diagram_objects = value

    gml_diagram_objects = property(get_gml_diagram_objects, set_gml_diagram_objects)

    def add_gml_diagram_objects(self, *gml_diagram_objects):
        for obj in gml_diagram_objects:
            if self not in obj._diagrams:
                obj._diagrams.append(self)
            self._gml_diagram_objects.append(obj)

    def remove_gml_diagram_objects(self, *gml_diagram_objects):
        for obj in gml_diagram_objects:
            if self in obj._diagrams:
                obj._diagrams.remove(self)
            self._gml_diagram_objects.remove(obj)
    # >>> gml_diagram_objects

    # <<< design_locations
    # @generated
    def get_design_locations(self):
        """ 
        """
        return self._design_locations

    def set_design_locations(self, value):
        for p in self._design_locations:
            filtered = [q for q in p.diagrams if q != self]
            self._design_locations._diagrams = filtered
        for r in value:
            if self not in r._diagrams:
                r._diagrams.append(self)
        self._design_locations = value

    design_locations = property(get_design_locations, set_design_locations)

    def add_design_locations(self, *design_locations):
        for obj in design_locations:
            if self not in obj._diagrams:
                obj._diagrams.append(self)
            self._design_locations.append(obj)

    def remove_design_locations(self, *design_locations):
        for obj in design_locations:
            if self in obj._diagrams:
                obj._diagrams.remove(self)
            self._design_locations.remove(obj)
    # >>> design_locations

    # <<< gml_coordinate_system
    # @generated
    def get_gml_coordinate_system(self):
        """ 
        """
        return self._gml_coordinate_system

    def set_gml_coordinate_system(self, value):
        if self._gml_coordinate_system is not None:
            filtered = [x for x in self.gml_coordinate_system.diagrams if x != self]
            self._gml_coordinate_system._diagrams = filtered

        self._gml_coordinate_system = value
        if self._gml_coordinate_system is not None:
            self._gml_coordinate_system._diagrams.append(self)

    gml_coordinate_system = property(get_gml_coordinate_system, set_gml_coordinate_system)
    # >>> gml_coordinate_system


    def __str__(self):
        """ Returns a string representation of the Diagram.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< diagram.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the Diagram.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "Diagram", self.uri)
        if format:
            indent += ' ' * depth

        for obj in self.gml_diagram_objects:
            s += '%s<%s:Diagram.gml_diagram_objects rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.design_locations:
            s += '%s<%s:Diagram.design_locations rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.gml_coordinate_system is not None:
            s += '%s<%s:Diagram.gml_coordinate_system rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.gml_coordinate_system.uri)
        s += '%s<%s:Diagram.kind>%s</%s:Diagram.kind>' % \
            (indent, ns_prefix, self.kind, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        for obj in self.activity_records:
            s += '%s<%s:Document.activity_records rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_organisation_roles:
            s += '%s<%s:Document.erp_organisation_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.scheduled_events:
            s += '%s<%s:Document.scheduled_events rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.from_document_roles:
            s += '%s<%s:Document.from_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.location_roles:
            s += '%s<%s:Document.location_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.power_system_resource_roles:
            s += '%s<%s:Document.power_system_resource_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.network_data_sets:
            s += '%s<%s:Document.network_data_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_person_roles:
            s += '%s<%s:Document.erp_person_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_items:
            s += '%s<%s:Document.change_items rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.measurements:
            s += '%s<%s:Document.measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.doc_status is not None:
            s += '%s<%s:Document.doc_status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.doc_status.uri)
        for obj in self.schedule_parameter_infos:
            s += '%s<%s:Document.schedule_parameter_infos rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.electronic_address is not None:
            s += '%s<%s:Document.electronic_address rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.electronic_address.uri)
        for obj in self.to_document_roles:
            s += '%s<%s:Document.to_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.status is not None:
            s += '%s<%s:Document.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        for obj in self.asset_roles:
            s += '%s<%s:Document.asset_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_sets:
            s += '%s<%s:Document.change_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:Document.subject>%s</%s:Document.subject>' % \
            (indent, ns_prefix, self.subject, ns_prefix)
        s += '%s<%s:Document.revision_number>%s</%s:Document.revision_number>' % \
            (indent, ns_prefix, self.revision_number, ns_prefix)
        s += '%s<%s:Document.category>%s</%s:Document.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        s += '%s<%s:Document.last_modified_date_time>%s</%s:Document.last_modified_date_time>' % \
            (indent, ns_prefix, self.last_modified_date_time, ns_prefix)
        s += '%s<%s:Document.title>%s</%s:Document.title>' % \
            (indent, ns_prefix, self.title, ns_prefix)
        s += '%s<%s:Document.created_date_time>%s</%s:Document.created_date_time>' % \
            (indent, ns_prefix, self.created_date_time, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "Diagram")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> diagram.serialize


class Ratio(Element):
    """ Fraction specified explicitly with a numerator and denominator, which can be used to calculate the quotient.
    """
    # <<< ratio
    # @generated
    def __init__(self, denominator=0.0, numerator=0.0, **kw_args):
        """ Initialises a new 'Ratio' instance.
        """
        # The part of a fraction that is below the line and that functions as the divisor of the numerator. 
        self.denominator = denominator

        # The part of a fraction that is above the line and signifies the number to be divided by the denominator. 
        self.numerator = numerator



        super(Ratio, self).__init__(**kw_args)
    # >>> ratio


    def __str__(self):
        """ Returns a string representation of the Ratio.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< ratio.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the Ratio.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "Ratio", self.uri)
        if format:
            indent += ' ' * depth

        s += '%s<%s:Ratio.denominator>%s</%s:Ratio.denominator>' % \
            (indent, ns_prefix, self.denominator, ns_prefix)
        s += '%s<%s:Ratio.numerator>%s</%s:Ratio.numerator>' % \
            (indent, ns_prefix, self.numerator, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "Ratio")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> ratio.serialize


class BusinessPlan(Document):
    """ A BusinessPlan is an organized sequence of predetermined actions required to complete a future organizational objective. It is a type of document that typically references a schedule, physical and/or logical resources (assets and/or PowerSystemResources), locations, etc.
    """
    pass
    # <<< business_plan
    # @generated
    def __init__(self, **kw_args):
        """ Initialises a new 'BusinessPlan' instance.
        """


        super(BusinessPlan, self).__init__(**kw_args)
    # >>> business_plan


    def __str__(self):
        """ Returns a string representation of the BusinessPlan.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< business_plan.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the BusinessPlan.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "BusinessPlan", self.uri)
        if format:
            indent += ' ' * depth

        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        for obj in self.activity_records:
            s += '%s<%s:Document.activity_records rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_organisation_roles:
            s += '%s<%s:Document.erp_organisation_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.scheduled_events:
            s += '%s<%s:Document.scheduled_events rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.from_document_roles:
            s += '%s<%s:Document.from_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.location_roles:
            s += '%s<%s:Document.location_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.power_system_resource_roles:
            s += '%s<%s:Document.power_system_resource_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.network_data_sets:
            s += '%s<%s:Document.network_data_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_person_roles:
            s += '%s<%s:Document.erp_person_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_items:
            s += '%s<%s:Document.change_items rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.measurements:
            s += '%s<%s:Document.measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.doc_status is not None:
            s += '%s<%s:Document.doc_status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.doc_status.uri)
        for obj in self.schedule_parameter_infos:
            s += '%s<%s:Document.schedule_parameter_infos rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.electronic_address is not None:
            s += '%s<%s:Document.electronic_address rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.electronic_address.uri)
        for obj in self.to_document_roles:
            s += '%s<%s:Document.to_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.status is not None:
            s += '%s<%s:Document.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        for obj in self.asset_roles:
            s += '%s<%s:Document.asset_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_sets:
            s += '%s<%s:Document.change_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:Document.subject>%s</%s:Document.subject>' % \
            (indent, ns_prefix, self.subject, ns_prefix)
        s += '%s<%s:Document.revision_number>%s</%s:Document.revision_number>' % \
            (indent, ns_prefix, self.revision_number, ns_prefix)
        s += '%s<%s:Document.category>%s</%s:Document.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        s += '%s<%s:Document.last_modified_date_time>%s</%s:Document.last_modified_date_time>' % \
            (indent, ns_prefix, self.last_modified_date_time, ns_prefix)
        s += '%s<%s:Document.title>%s</%s:Document.title>' % \
            (indent, ns_prefix, self.title, ns_prefix)
        s += '%s<%s:Document.created_date_time>%s</%s:Document.created_date_time>' % \
            (indent, ns_prefix, self.created_date_time, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "BusinessPlan")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> business_plan.serialize


class BusinessRole(IdentifiedObject):
    """ A business role that this organisation plays. A single organisation typically performs many functions, each one described as a role.
    """
    # <<< business_role
    # @generated
    def __init__(self, category='', organisations=None, status=None, **kw_args):
        """ Initialises a new 'BusinessRole' instance.
        """
        # Category by utility's corporate standards and practices. 
        self.category = category


        self._organisations = []
        if organisations is not None:
            self.organisations = organisations
        else:
            self.organisations = []

        self.status = status


        super(BusinessRole, self).__init__(**kw_args)
    # >>> business_role

    # <<< organisations
    # @generated
    def get_organisations(self):
        """ 
        """
        return self._organisations

    def set_organisations(self, value):
        for p in self._organisations:
            filtered = [q for q in p.business_roles if q != self]
            self._organisations._business_roles = filtered
        for r in value:
            if self not in r._business_roles:
                r._business_roles.append(self)
        self._organisations = value

    organisations = property(get_organisations, set_organisations)

    def add_organisations(self, *organisations):
        for obj in organisations:
            if self not in obj._business_roles:
                obj._business_roles.append(self)
            self._organisations.append(obj)

    def remove_organisations(self, *organisations):
        for obj in organisations:
            if self in obj._business_roles:
                obj._business_roles.remove(self)
            self._organisations.remove(obj)
    # >>> organisations

    # <<< status
    # @generated
    status = None
    # >>> status


    def __str__(self):
        """ Returns a string representation of the BusinessRole.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< business_role.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the BusinessRole.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "BusinessRole", self.uri)
        if format:
            indent += ' ' * depth

        for obj in self.organisations:
            s += '%s<%s:BusinessRole.organisations rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.status is not None:
            s += '%s<%s:BusinessRole.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        s += '%s<%s:BusinessRole.category>%s</%s:BusinessRole.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "BusinessRole")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> business_role.serialize


class Craft(IdentifiedObject):
    """ Craft of a person or a crew. Examples include overhead electric, underground electric, high pressure gas, etc. This ensures necessary knowledge and skills before being allowed to perform certain types of work.
    """
    # <<< craft
    # @generated
    def __init__(self, category='', skills=None, erp_persons=None, status=None, capabilities=None, **kw_args):
        """ Initialises a new 'Craft' instance.
        """
        # Category by utility's work mangement standards and practices. 
        self.category = category


        self._skills = []
        if skills is not None:
            self.skills = skills
        else:
            self.skills = []

        self._erp_persons = []
        if erp_persons is not None:
            self.erp_persons = erp_persons
        else:
            self.erp_persons = []

        self.status = status

        self._capabilities = []
        if capabilities is not None:
            self.capabilities = capabilities
        else:
            self.capabilities = []


        super(Craft, self).__init__(**kw_args)
    # >>> craft

    # <<< skills
    # @generated
    def get_skills(self):
        """ 
        """
        return self._skills

    def set_skills(self, value):
        for p in self._skills:
            filtered = [q for q in p.crafts if q != self]
            self._skills._crafts = filtered
        for r in value:
            if self not in r._crafts:
                r._crafts.append(self)
        self._skills = value

    skills = property(get_skills, set_skills)

    def add_skills(self, *skills):
        for obj in skills:
            if self not in obj._crafts:
                obj._crafts.append(self)
            self._skills.append(obj)

    def remove_skills(self, *skills):
        for obj in skills:
            if self in obj._crafts:
                obj._crafts.remove(self)
            self._skills.remove(obj)
    # >>> skills

    # <<< erp_persons
    # @generated
    def get_erp_persons(self):
        """ 
        """
        return self._erp_persons

    def set_erp_persons(self, value):
        for p in self._erp_persons:
            filtered = [q for q in p.crafts if q != self]
            self._erp_persons._crafts = filtered
        for r in value:
            if self not in r._crafts:
                r._crafts.append(self)
        self._erp_persons = value

    erp_persons = property(get_erp_persons, set_erp_persons)

    def add_erp_persons(self, *erp_persons):
        for obj in erp_persons:
            if self not in obj._crafts:
                obj._crafts.append(self)
            self._erp_persons.append(obj)

    def remove_erp_persons(self, *erp_persons):
        for obj in erp_persons:
            if self in obj._crafts:
                obj._crafts.remove(self)
            self._erp_persons.remove(obj)
    # >>> erp_persons

    # <<< status
    # @generated
    status = None
    # >>> status

    # <<< capabilities
    # @generated
    def get_capabilities(self):
        """ 
        """
        return self._capabilities

    def set_capabilities(self, value):
        for p in self._capabilities:
            filtered = [q for q in p.crafts if q != self]
            self._capabilities._crafts = filtered
        for r in value:
            if self not in r._crafts:
                r._crafts.append(self)
        self._capabilities = value

    capabilities = property(get_capabilities, set_capabilities)

    def add_capabilities(self, *capabilities):
        for obj in capabilities:
            if self not in obj._crafts:
                obj._crafts.append(self)
            self._capabilities.append(obj)

    def remove_capabilities(self, *capabilities):
        for obj in capabilities:
            if self in obj._crafts:
                obj._crafts.remove(self)
            self._capabilities.remove(obj)
    # >>> capabilities


    def __str__(self):
        """ Returns a string representation of the Craft.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< craft.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the Craft.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "Craft", self.uri)
        if format:
            indent += ' ' * depth

        for obj in self.skills:
            s += '%s<%s:Craft.skills rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_persons:
            s += '%s<%s:Craft.erp_persons rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.status is not None:
            s += '%s<%s:Craft.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        for obj in self.capabilities:
            s += '%s<%s:Craft.capabilities rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:Craft.category>%s</%s:Craft.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "Craft")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> craft.serialize


class ScheduleParameterInfo(IdentifiedObject):
    """ Schedule parameters for an activity that is to occur, is occurring, or has completed.
    """
    # <<< schedule_parameter_info
    # @generated
    def __init__(self, for_inspection_data_set=None, estimated_window=None, scheduled_events=None, status=None, documents=None, requested_window=None, **kw_args):
        """ Initialises a new 'ScheduleParameterInfo' instance.
        """

        self._for_inspection_data_set = None
        self.for_inspection_data_set = for_inspection_data_set

        self.estimated_window = estimated_window

        self._scheduled_events = []
        if scheduled_events is not None:
            self.scheduled_events = scheduled_events
        else:
            self.scheduled_events = []

        self.status = status

        self._documents = []
        if documents is not None:
            self.documents = documents
        else:
            self.documents = []

        self.requested_window = requested_window


        super(ScheduleParameterInfo, self).__init__(**kw_args)
    # >>> schedule_parameter_info

    # <<< for_inspection_data_set
    # @generated
    def get_for_inspection_data_set(self):
        """ 
        """
        return self._for_inspection_data_set

    def set_for_inspection_data_set(self, value):
        if self._for_inspection_data_set is not None:
            filtered = [x for x in self.for_inspection_data_set.according_to_schedules if x != self]
            self._for_inspection_data_set._according_to_schedules = filtered

        self._for_inspection_data_set = value
        if self._for_inspection_data_set is not None:
            self._for_inspection_data_set._according_to_schedules.append(self)

    for_inspection_data_set = property(get_for_inspection_data_set, set_for_inspection_data_set)
    # >>> for_inspection_data_set

    # <<< estimated_window
    # @generated
    # Estimated date and time for activity execution (with earliest possibility of activity initiation and latest possibility of activity completion).
    estimated_window = None
    # >>> estimated_window

    # <<< scheduled_events
    # @generated
    def get_scheduled_events(self):
        """ 
        """
        return self._scheduled_events

    def set_scheduled_events(self, value):
        for x in self._scheduled_events:
            x._schedule_parameter_info = None
        for y in value:
            y._schedule_parameter_info = self
        self._scheduled_events = value

    scheduled_events = property(get_scheduled_events, set_scheduled_events)

    def add_scheduled_events(self, *scheduled_events):
        for obj in scheduled_events:
            obj._schedule_parameter_info = self
            self._scheduled_events.append(obj)

    def remove_scheduled_events(self, *scheduled_events):
        for obj in scheduled_events:
            obj._schedule_parameter_info = None
            self._scheduled_events.remove(obj)
    # >>> scheduled_events

    # <<< status
    # @generated
    status = None
    # >>> status

    # <<< documents
    # @generated
    def get_documents(self):
        """ 
        """
        return self._documents

    def set_documents(self, value):
        for p in self._documents:
            filtered = [q for q in p.schedule_parameter_infos if q != self]
            self._documents._schedule_parameter_infos = filtered
        for r in value:
            if self not in r._schedule_parameter_infos:
                r._schedule_parameter_infos.append(self)
        self._documents = value

    documents = property(get_documents, set_documents)

    def add_documents(self, *documents):
        for obj in documents:
            if self not in obj._schedule_parameter_infos:
                obj._schedule_parameter_infos.append(self)
            self._documents.append(obj)

    def remove_documents(self, *documents):
        for obj in documents:
            if self in obj._schedule_parameter_infos:
                obj._schedule_parameter_infos.remove(self)
            self._documents.remove(obj)
    # >>> documents

    # <<< requested_window
    # @generated
    # Requested date and time interval for activity execution.
    requested_window = None
    # >>> requested_window


    def __str__(self):
        """ Returns a string representation of the ScheduleParameterInfo.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< schedule_parameter_info.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the ScheduleParameterInfo.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "ScheduleParameterInfo", self.uri)
        if format:
            indent += ' ' * depth

        if self.for_inspection_data_set is not None:
            s += '%s<%s:ScheduleParameterInfo.for_inspection_data_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.for_inspection_data_set.uri)
        if self.estimated_window is not None:
            s += '%s<%s:ScheduleParameterInfo.estimated_window rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.estimated_window.uri)
        for obj in self.scheduled_events:
            s += '%s<%s:ScheduleParameterInfo.scheduled_events rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.status is not None:
            s += '%s<%s:ScheduleParameterInfo.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        for obj in self.documents:
            s += '%s<%s:ScheduleParameterInfo.documents rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.requested_window is not None:
            s += '%s<%s:ScheduleParameterInfo.requested_window rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.requested_window.uri)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "ScheduleParameterInfo")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> schedule_parameter_info.serialize


class Map(Diagram):
    """ A type of diagram that is usually printed on paper. It normally depicts part of the earth's surface, showing utility assets, right of ways, topological data, coordinates, grids, etc. Maps vary depending on whether they are used for dispatch, design, schematic, etc.
    """
    # <<< map
    # @generated
    def __init__(self, map_loc_grid='', page_number=0, **kw_args):
        """ Initialises a new 'Map' instance.
        """
        # Map grid number. 
        self.map_loc_grid = map_loc_grid

        # Page number for particular set of maps specified by 'category'. 
        self.page_number = page_number



        super(Map, self).__init__(**kw_args)
    # >>> map


    def __str__(self):
        """ Returns a string representation of the Map.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< map.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the Map.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "Map", self.uri)
        if format:
            indent += ' ' * depth

        s += '%s<%s:Map.map_loc_grid>%s</%s:Map.map_loc_grid>' % \
            (indent, ns_prefix, self.map_loc_grid, ns_prefix)
        s += '%s<%s:Map.page_number>%s</%s:Map.page_number>' % \
            (indent, ns_prefix, self.page_number, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        for obj in self.activity_records:
            s += '%s<%s:Document.activity_records rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_organisation_roles:
            s += '%s<%s:Document.erp_organisation_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.scheduled_events:
            s += '%s<%s:Document.scheduled_events rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.from_document_roles:
            s += '%s<%s:Document.from_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.location_roles:
            s += '%s<%s:Document.location_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.power_system_resource_roles:
            s += '%s<%s:Document.power_system_resource_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.network_data_sets:
            s += '%s<%s:Document.network_data_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_person_roles:
            s += '%s<%s:Document.erp_person_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_items:
            s += '%s<%s:Document.change_items rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.measurements:
            s += '%s<%s:Document.measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.doc_status is not None:
            s += '%s<%s:Document.doc_status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.doc_status.uri)
        for obj in self.schedule_parameter_infos:
            s += '%s<%s:Document.schedule_parameter_infos rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.electronic_address is not None:
            s += '%s<%s:Document.electronic_address rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.electronic_address.uri)
        for obj in self.to_document_roles:
            s += '%s<%s:Document.to_document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.status is not None:
            s += '%s<%s:Document.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        for obj in self.asset_roles:
            s += '%s<%s:Document.asset_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_sets:
            s += '%s<%s:Document.change_sets rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:Document.subject>%s</%s:Document.subject>' % \
            (indent, ns_prefix, self.subject, ns_prefix)
        s += '%s<%s:Document.revision_number>%s</%s:Document.revision_number>' % \
            (indent, ns_prefix, self.revision_number, ns_prefix)
        s += '%s<%s:Document.category>%s</%s:Document.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        s += '%s<%s:Document.last_modified_date_time>%s</%s:Document.last_modified_date_time>' % \
            (indent, ns_prefix, self.last_modified_date_time, ns_prefix)
        s += '%s<%s:Document.title>%s</%s:Document.title>' % \
            (indent, ns_prefix, self.title, ns_prefix)
        s += '%s<%s:Document.created_date_time>%s</%s:Document.created_date_time>' % \
            (indent, ns_prefix, self.created_date_time, ns_prefix)
        for obj in self.gml_diagram_objects:
            s += '%s<%s:Diagram.gml_diagram_objects rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.design_locations:
            s += '%s<%s:Diagram.design_locations rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.gml_coordinate_system is not None:
            s += '%s<%s:Diagram.gml_coordinate_system rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.gml_coordinate_system.uri)
        s += '%s<%s:Diagram.kind>%s</%s:Diagram.kind>' % \
            (indent, ns_prefix, self.kind, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "Map")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> map.serialize


class DocPsrRole(Role):
    """ Potential roles that might played by a document relative to a type of PowerSystemResource.
    """
    # <<< doc_psr_role
    # @generated
    def __init__(self, document=None, power_system_resource=None, **kw_args):
        """ Initialises a new 'DocPsrRole' instance.
        """

        self._document = None
        self.document = document

        self._power_system_resource = None
        self.power_system_resource = power_system_resource


        super(DocPsrRole, self).__init__(**kw_args)
    # >>> doc_psr_role

    # <<< document
    # @generated
    def get_document(self):
        """ 
        """
        return self._document

    def set_document(self, value):
        if self._document is not None:
            filtered = [x for x in self.document.power_system_resource_roles if x != self]
            self._document._power_system_resource_roles = filtered

        self._document = value
        if self._document is not None:
            self._document._power_system_resource_roles.append(self)

    document = property(get_document, set_document)
    # >>> document

    # <<< power_system_resource
    # @generated
    def get_power_system_resource(self):
        """ 
        """
        return self._power_system_resource

    def set_power_system_resource(self, value):
        if self._power_system_resource is not None:
            filtered = [x for x in self.power_system_resource.document_roles if x != self]
            self._power_system_resource._document_roles = filtered

        self._power_system_resource = value
        if self._power_system_resource is not None:
            self._power_system_resource._document_roles.append(self)

    power_system_resource = property(get_power_system_resource, set_power_system_resource)
    # >>> power_system_resource


    def __str__(self):
        """ Returns a string representation of the DocPsrRole.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< doc_psr_role.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the DocPsrRole.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "DocPsrRole", self.uri)
        if format:
            indent += ' ' * depth

        if self.document is not None:
            s += '%s<%s:DocPsrRole.document rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.document.uri)
        if self.power_system_resource is not None:
            s += '%s<%s:DocPsrRole.power_system_resource rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.power_system_resource.uri)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        if self.status is not None:
            s += '%s<%s:Role.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        s += '%s<%s:Role.category>%s</%s:Role.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "DocPsrRole")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> doc_psr_role.serialize


class DocDocRole(Role):
    """ Roles played between Documents and other Documents.
    """
    # <<< doc_doc_role
    # @generated
    def __init__(self, to_document=None, from_document=None, **kw_args):
        """ Initialises a new 'DocDocRole' instance.
        """

        self._to_document = None
        self.to_document = to_document

        self._from_document = None
        self.from_document = from_document


        super(DocDocRole, self).__init__(**kw_args)
    # >>> doc_doc_role

    # <<< to_document
    # @generated
    def get_to_document(self):
        """ 
        """
        return self._to_document

    def set_to_document(self, value):
        if self._to_document is not None:
            filtered = [x for x in self.to_document.from_document_roles if x != self]
            self._to_document._from_document_roles = filtered

        self._to_document = value
        if self._to_document is not None:
            self._to_document._from_document_roles.append(self)

    to_document = property(get_to_document, set_to_document)
    # >>> to_document

    # <<< from_document
    # @generated
    def get_from_document(self):
        """ 
        """
        return self._from_document

    def set_from_document(self, value):
        if self._from_document is not None:
            filtered = [x for x in self.from_document.to_document_roles if x != self]
            self._from_document._to_document_roles = filtered

        self._from_document = value
        if self._from_document is not None:
            self._from_document._to_document_roles.append(self)

    from_document = property(get_from_document, set_from_document)
    # >>> from_document


    def __str__(self):
        """ Returns a string representation of the DocDocRole.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< doc_doc_role.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the DocDocRole.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "DocDocRole", self.uri)
        if format:
            indent += ' ' * depth

        if self.to_document is not None:
            s += '%s<%s:DocDocRole.to_document rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.to_document.uri)
        if self.from_document is not None:
            s += '%s<%s:DocDocRole.from_document rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.from_document.uri)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        if self.status is not None:
            s += '%s<%s:Role.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        s += '%s<%s:Role.category>%s</%s:Role.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "DocDocRole")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> doc_doc_role.serialize


# <<< inf_common
# @generated
# >>> inf_common



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CIM import Element
from CIM.IEC61968.Common import Document
from CIM.IEC61968.Common import Agreement
from CIM.IEC61968.Common import Organisation
from CIM.IEC61968.Metering import DeviceFunction
from CIM.IEC61970.Core import IdentifiedObject
from CIM.IEC61968.PaymentMetering import TenderKind
from CIM.IEC61970.Domain import Money
from CIM.IEC61968.PaymentMetering import TransactionKind
from CIM.IEC61968.PaymentMetering import CreditKind
from CIM.IEC61968.PaymentMetering import ChargeKind



from enthought.traits.api import Instance, List, Property, Bool, Date, Str
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
#  "ReceiptSummary" class:
#------------------------------------------------------------------------------

class ReceiptSummary(Element):
    """ Record of detail of receipts pertaining to one shift of operation (one record per 'tenderKind').
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Shift for which this summary is given.
    Shift = Instance("CIM.IEC61968.PaymentMetering.Shift",
        desc="Shift for which this summary is given.",
        transient=True,
        opposite="ReceiptSummaries",
        editor=InstanceEditor(name="_shifts"))

    def _get_shifts(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.Shift" ]
        else:
            return []

    _shifts = Property(fget=_get_shifts)

    # Totalised amount receipted during the shift for 'tenderKind', i.e., sum of ('Tender.amount' - 'Tender.change') per 'Tender.kind'.
    line = Instance("CIM.IEC61968.PaymentMetering.LineDetail",
        desc="Totalised amount receipted during the shift for 'tenderKind', i.e., sum of ('Tender.amount' - 'Tender.change') per 'Tender.kind'.",
        transient=True,
        editor=InstanceEditor(name="_linedetails"))

    def _get_linedetails(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.LineDetail" ]
        else:
            return []

    _linedetails = Property(fget=_get_linedetails)

    # 'Tender.kind' for which 'receiptsTotal' is given.
    tenderKind = TenderKind(desc="'Tender.kind' for which 'receiptsTotal' is given.")

    #--------------------------------------------------------------------------
    #  Begin "ReceiptSummary" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "tenderKind",
                label="Attributes"),
            VGroup("Parent", "Shift", "line",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfPaymentMetering.ReceiptSummary",
        title="ReceiptSummary",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ReceiptSummary" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "BankStatement" class:
#------------------------------------------------------------------------------

class BankStatement(Document):
    """ A type of Document that records bank deposits made by Vendor of revenue collected at PointOfSale.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The Vendor that made this BankStatement (by making deposit).
    Vendor = Instance("CIM.IEC61968.PaymentMetering.Vendor",
        desc="The Vendor that made this BankStatement (by making deposit).",
        transient=True,
        opposite="BankStatements",
        editor=InstanceEditor(name="_vendors"))

    def _get_vendors(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.Vendor" ]
        else:
            return []

    _vendors = Property(fget=_get_vendors)

    MerchantAccount = Instance("CIM.IEC61968.PaymentMetering.MerchantAccount",
        transient=True,
        opposite="BankStatements",
        editor=InstanceEditor(name="_merchantaccounts"))

    def _get_merchantaccounts(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.MerchantAccount" ]
        else:
            return []

    _merchantaccounts = Property(fget=_get_merchantaccounts)

    # BankAccount that generated this bank statement.
    BankAccount = Instance("CIM.IEC61968.Informative.InfCommon.BankAccount",
        desc="BankAccount that generated this bank statement.",
        transient=True,
        opposite="BankStatements",
        editor=InstanceEditor(name="_bankaccounts"))

    def _get_bankaccounts(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfCommon.BankAccount" ]
        else:
            return []

    _bankaccounts = Property(fget=_get_bankaccounts)

    # The amount that is deposited into the bank via BankAccount.
    depositAmount = Money(desc="The amount that is deposited into the bank via BankAccount.")

    # True if mechantCreditAmount has been cerdited to MerchantAccount; typically happens when bank statement details are captured into payment system.
    posted = Bool(desc="True if mechantCreditAmount has been cerdited to MerchantAccount; typically happens when bank statement details are captured into payment system.")

    # The date and time the deposit is made.
    depositDateTime = Date(desc="The date and time the deposit is made.")

    # The amount on this statement that is to be credited to MerchantAccount.
    merchantCreditAmount = Money(desc="The amount on this statement that is to be credited to MerchantAccount.")

    #--------------------------------------------------------------------------
    #  Begin "BankStatement" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "depositAmount", "posted", "depositDateTime", "merchantCreditAmount",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "Vendor", "MerchantAccount", "BankAccount",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfPaymentMetering.BankStatement",
        title="BankStatement",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "BankStatement" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TSPAgreement" class:
#------------------------------------------------------------------------------

class TSPAgreement(Agreement):
    """ A contractual agreement between a supplier (utility) and a transaction service provider (a type of organisation) that governs the terms and conditions, under which authorisation the transaction service provider may process transactions on behalf of the supplier.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "TSPAgreement" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "signDate",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "validityInterval",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfPaymentMetering.TSPAgreement",
        title="TSPAgreement",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TSPAgreement" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TransactionSummary" class:
#------------------------------------------------------------------------------

class TransactionSummary(Element):
    """ The record of detail of payment transactions pertaining to one shift of operation (one record per 'transactionKind').
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Shift to which this summary applies.
    Shift = Instance("CIM.IEC61968.PaymentMetering.Shift",
        desc="Shift to which this summary applies.",
        transient=True,
        opposite="TransactionSummaries",
        editor=InstanceEditor(name="_shifts"))

    def _get_shifts(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.Shift" ]
        else:
            return []

    _shifts = Property(fget=_get_shifts)

    # Totalised amount transacted during the shift for this specific 'transactionKind', i.e., sum of 'Transaction.line.amount' per 'Transaction.kind'. Cumulative amount of rounding errors due to process rounding not reflected in 'LineDetail.amount' for 'transactionKind', i.e., sum of 'Transaction.line.rounding' per 'Transaction.kind'.
    line = Instance("CIM.IEC61968.PaymentMetering.LineDetail",
        desc="Totalised amount transacted during the shift for this specific 'transactionKind', i.e., sum of 'Transaction.line.amount' per 'Transaction.kind'. Cumulative amount of rounding errors due to process rounding not reflected in 'LineDetail.amount' for 'transactionKind', i.e., sum of 'Transaction.line.rounding' per 'Transaction.kind'.",
        transient=True,
        editor=InstanceEditor(name="_linedetails"))

    def _get_linedetails(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.LineDetail" ]
        else:
            return []

    _linedetails = Property(fget=_get_linedetails)

    # 'Transaction.kind' for which 'transactionsTotal' is given.
    transactionKind = TransactionKind(desc="'Transaction.kind' for which 'transactionsTotal' is given.")

    #--------------------------------------------------------------------------
    #  Begin "TransactionSummary" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "transactionKind",
                label="Attributes"),
            VGroup("Parent", "Shift", "line",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfPaymentMetering.TransactionSummary",
        title="TransactionSummary",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TransactionSummary" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Bank" class:
#------------------------------------------------------------------------------

class Bank(Organisation):
    """ Organisation that is a commercial bank, agency, or other institution that offers a similar service.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All BankAccounts this Bank provides.
    BankAccounts = List(Instance("CIM.IEC61968.Informative.InfCommon.BankAccount"),
        desc="All BankAccounts this Bank provides.")

    # International bank account number defined in ISO 13616; for countries where IBAN is not in operation, the existing BIC or SWIFT codes may be used instead (see ISO 9362).
    iban = Str(desc="International bank account number defined in ISO 13616; for countries where IBAN is not in operation, the existing BIC or SWIFT codes may be used instead (see ISO 9362).")

    # Codified reference to the particular branch of the bank where BankAccount is held.
    branchCode = Str(desc="Codified reference to the particular branch of the bank where BankAccount is held.")

    # Bank identifier code as defined in ISO 9362; for use in countries wher IBAN is not yet in operation.
    bic = Str(desc="Bank identifier code as defined in ISO 9362; for use in countries wher IBAN is not yet in operation.")

    #--------------------------------------------------------------------------
    #  Begin "Bank" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "iban", "branchCode", "bic",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "BusinessRoles", "TelephoneNumbers", "streetAddress", "MarketRoles", "postalAddress", "ElectronicAddresses", "BankAccounts",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfPaymentMetering.Bank",
        title="Bank",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Bank" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SDPAccountingFunction" class:
#------------------------------------------------------------------------------

class SDPAccountingFunction(DeviceFunction):
    """ Service delivery point accounting function, particularly for payment meter.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ChargeRegisters = List(Instance("CIM.IEC61968.Informative.InfPaymentMetering.ChargeRegister"))

    # The value is the balance of the sum of credits minus the sum of charges from the CreditRegisters and the ChargeRegisters. The sum might be complex function. The units are either in currency units or service units, depending on the value of accountingMode.
    availableCredit = Instance("CIM.IEC61968.PaymentMetering.AccountingUnit",
        desc="The value is the balance of the sum of credits minus the sum of charges from the CreditRegisters and the ChargeRegisters. The sum might be complex function. The units are either in currency units or service units, depending on the value of accountingMode.",
        transient=True,
        editor=InstanceEditor(name="_accountingunits"))

    def _get_accountingunits(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.AccountingUnit" ]
        else:
            return []

    _accountingunits = Property(fget=_get_accountingunits)

    # The value is a predefined set point for the level of the availableCredit to reach when the service will be interrupted. In the case of a prepayment meter the interruption is typically implemented by means of an electro-mechanical switch and the value is typically set = 0. The units are either in currency units or service units, depending on the value of accountingMode.
    creditExpiryLevel = Instance("CIM.IEC61968.PaymentMetering.AccountingUnit",
        desc="The value is a predefined set point for the level of the availableCredit to reach when the service will be interrupted. In the case of a prepayment meter the interruption is typically implemented by means of an electro-mechanical switch and the value is typically set = 0. The units are either in currency units or service units, depending on the value of accountingMode.",
        transient=True,
        editor=InstanceEditor(name="_accountingunits"))

    def _get_accountingunits(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.AccountingUnit" ]
        else:
            return []

    _accountingunits = Property(fget=_get_accountingunits)

    CreditRegisters = List(Instance("CIM.IEC61968.Informative.InfPaymentMetering.CreditRegister"))

    ServiceKind = Instance("CIM.IEC61968.Customers.ServiceCategory",
        transient=True,
        opposite="SPAccountingFunctions",
        editor=InstanceEditor(name="_servicecategorys"))

    def _get_servicecategorys(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Customers.ServiceCategory" ]
        else:
            return []

    _servicecategorys = Property(fget=_get_servicecategorys)

    # The value is a predefined set point for the level of the availableCredit to reach when a warning will be indicated to the customer. It serves to warn the customer that it is time to purchase more credit in the case of a prepayment meter implementation. The units are either in currency units or service units, depending on the value of accountingMode.
    lowCreditWarningLevel = Instance("CIM.IEC61968.PaymentMetering.AccountingUnit",
        desc="The value is a predefined set point for the level of the availableCredit to reach when a warning will be indicated to the customer. It serves to warn the customer that it is time to purchase more credit in the case of a prepayment meter implementation. The units are either in currency units or service units, depending on the value of accountingMode.",
        transient=True,
        editor=InstanceEditor(name="_accountingunits"))

    def _get_accountingunits(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.AccountingUnit" ]
        else:
            return []

    _accountingunits = Property(fget=_get_accountingunits)

    #--------------------------------------------------------------------------
    #  Begin "SDPAccountingFunction" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "initialCondition", "category", "lotNumber", "application", "serialNumber", "installationDate", "corporateCode", "purchasePrice", "manufacturedDate", "initialLossOfLife", "utcNumber", "critical", "hardwareID", "configID", "programID", "password", "firmwareID", "disabled",
                label="Attributes", columns=2),
            VGroup("Parent", "ModelingAuthoritySet", "Measurements", "Hazards", "ErpOrganisationRoles", "DimensionsInfo", "ScheduledEvents", "Mediums", "AssetFunctions", "Properties", "AssetContainer", "Ratings", "ActivityRecords", "FromAssetRoles", "LocationRoles", "PowerSystemResourceRoles", "DocumentRoles", "ChangeItems", "ErpItemMaster", "ElectronicAddresses", "WorkTask", "ErpRecDeliveryItems", "ReliabilityInfos", "ToAssetRoles", "AssetPropertyCurves", "FinancialInfo", "ErpInventory", "acceptanceTest", "status", "Asset", "AssetFunctionAssetModel", "ComEquipmentAsset", "EndDeviceAsset", "Registers", "EndDeviceEvents", "ChargeRegisters", "availableCredit", "creditExpiryLevel", "CreditRegisters", "ServiceKind", "lowCreditWarningLevel",
                label="References", columns=4),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfPaymentMetering.SDPAccountingFunction",
        title="SDPAccountingFunction",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SDPAccountingFunction" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CreditRegister" class:
#------------------------------------------------------------------------------

class CreditRegister(IdentifiedObject):
    """ Accumulated credits transacted per CreditKind for a given function. There could be several of these registers, one for each CreditKind; depending on the application.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Credit amount in favour of the customer. The units are either in currency units or service units, depending on the value of 'AccountingUnit.accountingMode'.
    creditAmount = Instance("CIM.IEC61968.PaymentMetering.AccountingUnit",
        desc="Credit amount in favour of the customer. The units are either in currency units or service units, depending on the value of 'AccountingUnit.accountingMode'.",
        transient=True,
        editor=InstanceEditor(name="_accountingunits"))

    def _get_accountingunits(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.AccountingUnit" ]
        else:
            return []

    _accountingunits = Property(fget=_get_accountingunits)

    SDPAccountingFunction = Instance("CIM.IEC61968.Informative.InfPaymentMetering.SDPAccountingFunction",
        transient=True,
        opposite="CreditRegisters",
        editor=InstanceEditor(name="_sdpaccountingfunctions"))

    def _get_sdpaccountingfunctions(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfPaymentMetering.SDPAccountingFunction" ]
        else:
            return []

    _sdpaccountingfunctions = Property(fget=_get_sdpaccountingfunctions)

    # Several different types of credit are typically implemented in the case of a prepayment meter.  For example: credit transferred by means of a token carrier, or credit advanced automatically inside the meter under certain conditions, or credit held in reserved to be released under emergency conditions, or credit granted by local authority as a basic life support mechanism and may be dispensed automatically by the meter under certain conditions or credit available under severe climate conditions such as during winter over a weekend.
    creditKind = CreditKind(desc="Several different types of credit are typically implemented in the case of a prepayment meter.  For example: credit transferred by means of a token carrier, or credit advanced automatically inside the meter under certain conditions, or credit held in reserved to be released under emergency conditions, or credit granted by local authority as a basic life support mechanism and may be dispensed automatically by the meter under certain conditions or credit available under severe climate conditions such as during winter over a weekend.")

    #--------------------------------------------------------------------------
    #  Begin "CreditRegister" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "creditKind",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "creditAmount", "SDPAccountingFunction",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfPaymentMetering.CreditRegister",
        title="CreditRegister",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CreditRegister" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Token" class:
#------------------------------------------------------------------------------

class Token(IdentifiedObject):
    """ The token that is transferred to the payment meter.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # PointOfSale tha sold or dispensed this Token.
    PointOfSale = Instance("CIM.IEC61968.PaymentMetering.PointOfSale",
        desc="PointOfSale tha sold or dispensed this Token.",
        transient=True,
        opposite="Tokens",
        editor=InstanceEditor(name="_pointofsales"))

    def _get_pointofsales(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.PointOfSale" ]
        else:
            return []

    _pointofsales = Property(fget=_get_pointofsales)

    # Coded representation of the token that is transferred to the payment meter.
    code = Str(desc="Coded representation of the token that is transferred to the payment meter.")

    # Free-format note relevant to this token.
    comment = Str(desc="Free-format note relevant to this token.")

    #--------------------------------------------------------------------------
    #  Begin "Token" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "code", "comment",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "PointOfSale",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfPaymentMetering.Token",
        title="Token",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Token" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ChargeRegister" class:
#------------------------------------------------------------------------------

class ChargeRegister(IdentifiedObject):
    """ Accumulated charges transacted per ChargeKind for a given function. There could be several of these registers, one for each ChargeKind; depending on the application.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Charge amount in favour of the supplier. The units are either in currency units or service units, depending on the value of 'AccountingUnit.accountingMode'.
    chargeAmount = Instance("CIM.IEC61968.PaymentMetering.AccountingUnit",
        desc="Charge amount in favour of the supplier. The units are either in currency units or service units, depending on the value of 'AccountingUnit.accountingMode'.",
        transient=True,
        editor=InstanceEditor(name="_accountingunits"))

    def _get_accountingunits(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.PaymentMetering.AccountingUnit" ]
        else:
            return []

    _accountingunits = Property(fget=_get_accountingunits)

    SPAccountingFunction = Instance("CIM.IEC61968.Informative.InfPaymentMetering.SDPAccountingFunction",
        transient=True,
        opposite="ChargeRegisters",
        editor=InstanceEditor(name="_sdpaccountingfunctions"))

    def _get_sdpaccountingfunctions(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfPaymentMetering.SDPAccountingFunction" ]
        else:
            return []

    _sdpaccountingfunctions = Property(fget=_get_sdpaccountingfunctions)

    # Several different types of charges are typically implemented in the case of a prepayment meter. For example: a charge according to a tariff for consumption and possibly a demand component, or a charge for a debt that is loaded in the meter to be recovered on a time basis, or a standing charge to be levied at the end of each billing period, or a tax charge loaded in the meter to be recovered on a consumption basis or a time basis.
    chargeKind = ChargeKind(desc="Several different types of charges are typically implemented in the case of a prepayment meter. For example: a charge according to a tariff for consumption and possibly a demand component, or a charge for a debt that is loaded in the meter to be recovered on a time basis, or a standing charge to be levied at the end of each billing period, or a tax charge loaded in the meter to be recovered on a consumption basis or a time basis.")

    #--------------------------------------------------------------------------
    #  Begin "ChargeRegister" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "chargeKind",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "chargeAmount", "SPAccountingFunction",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfPaymentMetering.ChargeRegister",
        title="ChargeRegister",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ChargeRegister" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


""" Defines a map of class names to their module.
"""

ucte_pkg_map = {
    "IEC61970CIMVersion": "ucte",
    "Element": "ucte",
    "IdentifiedObject": "ucte.core",
    "PowerSystemResource": "ucte.core",
    "Equipment": "ucte.core",
    "ConductingEquipment": "ucte.core",
    "Curve": "ucte.core",
    "BasicIntervalSchedule": "ucte.core",
    "IrregularIntervalSchedule": "ucte.core",
    "RegularIntervalSchedule": "ucte.core",
    "ConnectivityNodeContainer": "ucte.core",
    "EquipmentContainer": "ucte.core",
    "CurveData": "ucte.core",
    "Bay": "ucte.core",
    "PSRType": "ucte.core",
    "GeographicalRegion": "ucte.core",
    "Terminal": "ucte.core",
    "OperatingParticipant": "ucte.core",
    "VoltageLevel": "ucte.core",
    "BasePower": "ucte.core",
    "Unit": "ucte.core",
    "ModelingAuthority": "ucte.core",
    "BaseVoltage": "ucte.core",
    "SubGeographicalRegion": "ucte.core",
    "PsrList": "ucte.core",
    "Substation": "ucte.core",
    "ReportingGroup": "ucte.core",
    "Company": "ucte.core",
    "ReportingSuperGroup": "ucte.core",
    "RegularTimePoint": "ucte.core",
    "IrregularTimePoint": "ucte.core",
    "ModelingAuthoritySet": "ucte.core",
    "OperatingShare": "ucte.core",
    "PowerTransformer": "ucte.wires",
    "RegulatingCondEq": "ucte.wires",
    "FrequencyConverter": "ucte.wires",
    "ShuntCompensator": "ucte.wires",
    "HeatExchanger": "ucte.wires",
    "RegulatingControl": "ucte.wires",
    "ReactiveCapabilityCurve": "ucte.wires",
    "Line": "ucte.wires",
    "Connector": "ucte.wires",
    "Junction": "ucte.wires",
    "Ground": "ucte.wires",
    "Conductor": "ucte.wires",
    "TransformerWinding": "ucte.wires",
    "WireArrangement": "ucte.wires",
    "EnergyConsumer": "ucte.wires",
    "Switch": "ucte.wires",
    "ProtectedSwitch": "ucte.wires",
    "LoadBreakSwitch": "ucte.wires",
    "ACLineSegment": "ucte.wires",
    "Plant": "ucte.wires",
    "WireType": "ucte.wires",
    "RegulationSchedule": "ucte.wires",
    "WindingTest": "ucte.wires",
    "MutualCoupling": "ucte.wires",
    "Disconnector": "ucte.wires",
    "SeriesCompensator": "ucte.wires",
    "GroundDisconnector": "ucte.wires",
    "SynchronousMachine": "ucte.wires",
    "CompositeSwitch": "ucte.wires",
    "TapChanger": "ucte.wires",
    "PhaseTapChanger": "ucte.wires",
    "RectifierInverter": "ucte.wires",
    "StaticVarCompensator": "ucte.wires",
    "RatioTapChanger": "ucte.wires",
    "ConductorType": "ucte.wires",
    "VoltageControlZone": "ucte.wires",
    "EnergySource": "ucte.wires",
    "Fuse": "ucte.wires",
    "Jumper": "ucte.wires",
    "DCLineSegment": "ucte.wires",
    "Breaker": "ucte.wires",
    "BusbarSection": "ucte.wires",
    "TopologicalIsland": "ucte.topology",
    "ConnectivityNode": "ucte.topology",
    "TopologicalNode": "ucte.topology",
    "BusNameMarker": "ucte.topology",
    "ClearanceTag": "ucte.outage",
    "ClearanceTagType": "ucte.outage",
    "OutageSchedule": "ucte.outage",
    "SwitchingOperation": "ucte.outage",
    "SteamSupply": "ucte.generation.generation_dynamics",
    "FossilSteamSupply": "ucte.generation.generation_dynamics",
    "HeatRecoveryBoiler": "ucte.generation.generation_dynamics",
    "PWRSteamSupply": "ucte.generation.generation_dynamics",
    "PrimeMover": "ucte.generation.generation_dynamics",
    "Supercritical": "ucte.generation.generation_dynamics",
    "CombustionTurbine": "ucte.generation.generation_dynamics",
    "HydroTurbine": "ucte.generation.generation_dynamics",
    "Subcritical": "ucte.generation.generation_dynamics",
    "CTTempActivePowerCurve": "ucte.generation.generation_dynamics",
    "SteamTurbine": "ucte.generation.generation_dynamics",
    "DrumBoiler": "ucte.generation.generation_dynamics",
    "BWRSteamSupply": "ucte.generation.generation_dynamics",
    "LevelVsVolumeCurve": "ucte.generation.production",
    "FossilFuel": "ucte.generation.production",
    "SteamSendoutSchedule": "ucte.generation.production",
    "EmissionCurve": "ucte.generation.production",
    "CombinedCyclePlant": "ucte.generation.production",
    "StartIgnFuelCurve": "ucte.generation.production",
    "HydroGeneratingEfficiencyCurve": "ucte.generation.production",
    "StartRampCurve": "ucte.generation.production",
    "GeneratingUnit": "ucte.generation.production",
    "NuclearGeneratingUnit": "ucte.generation.production",
    "WindGeneratingUnit": "ucte.generation.production",
    "StartMainFuelCurve": "ucte.generation.production",
    "StartupModel": "ucte.generation.production",
    "AirCompressor": "ucte.generation.production",
    "HeatInputCurve": "ucte.generation.production",
    "CogenerationPlant": "ucte.generation.production",
    "ShutdownCurve": "ucte.generation.production",
    "InflowForecast": "ucte.generation.production",
    "TargetLevelSchedule": "ucte.generation.production",
    "HydroGeneratingUnit": "ucte.generation.production",
    "EmissionAccount": "ucte.generation.production",
    "GrossToNetActivePowerCurve": "ucte.generation.production",
    "HydroPumpOpSchedule": "ucte.generation.production",
    "Reservoir": "ucte.generation.production",
    "CAESPlant": "ucte.generation.production",
    "GenUnitOpCostCurve": "ucte.generation.production",
    "PenstockLossCurve": "ucte.generation.production",
    "HydroPump": "ucte.generation.production",
    "GenUnitOpSchedule": "ucte.generation.production",
    "FuelAllocationSchedule": "ucte.generation.production",
    "HeatRateCurve": "ucte.generation.production",
    "IncrementalHeatRateCurve": "ucte.generation.production",
    "ThermalGeneratingUnit": "ucte.generation.production",
    "TailbayLossCurve": "ucte.generation.production",
    "HydroPowerPlant": "ucte.generation.production",
    "PowerCutZone": "ucte.load_model",
    "LoadResponseCharacteristic": "ucte.load_model",
    "EnergyArea": "ucte.load_model",
    "LoadArea": "ucte.load_model",
    "StationSupply": "ucte.load_model",
    "SubLoadArea": "ucte.load_model",
    "ConformLoad": "ucte.load_model",
    "Load": "ucte.load_model",
    "NonConformLoad": "ucte.load_model",
    "InductionMotorLoad": "ucte.load_model",
    "LoadGroup": "ucte.load_model",
    "NonConformLoadGroup": "ucte.load_model",
    "Season": "ucte.load_model",
    "SeasonDayTypeSchedule": "ucte.load_model",
    "ConformLoadSchedule": "ucte.load_model",
    "NonConformLoadSchedule": "ucte.load_model",
    "CustomerLoad": "ucte.load_model",
    "DayType": "ucte.load_model",
    "ConformLoadGroup": "ucte.load_model",
    "OperationalLimit": "ucte.operational_limits",
    "CurrentLimit": "ucte.operational_limits",
    "BranchGroup": "ucte.operational_limits",
    "BranchGroupTerminal": "ucte.operational_limits",
    "ApparentPowerLimit": "ucte.operational_limits",
    "OperationalLimitSet": "ucte.operational_limits",
    "VoltageLimit": "ucte.operational_limits",
    "ActivePowerLimit": "ucte.operational_limits",
    "OperationalLimitType": "ucte.operational_limits",
    "Control": "ucte.meas",
    "Measurement": "ucte.meas",
    "StringMeasurement": "ucte.meas",
    "Discrete": "ucte.meas",
    "ValueAliasSet": "ucte.meas",
    "MeasurementValue": "ucte.meas",
    "DiscreteValue": "ucte.meas",
    "Limit": "ucte.meas",
    "AnalogLimit": "ucte.meas",
    "LimitSet": "ucte.meas",
    "AccumulatorLimitSet": "ucte.meas",
    "SetPoint": "ucte.meas",
    "Command": "ucte.meas",
    "StringMeasurementValue": "ucte.meas",
    "ValueToAlias": "ucte.meas",
    "ControlType": "ucte.meas",
    "AnalogLimitSet": "ucte.meas",
    "Accumulator": "ucte.meas",
    "AccumulatorLimit": "ucte.meas",
    "MeasurementValueSource": "ucte.meas",
    "AnalogValue": "ucte.meas",
    "Analog": "ucte.meas",
    "MeasurementType": "ucte.meas",
    "Quality61850": "ucte.meas",
    "MeasurementValueQuality": "ucte.meas",
    "AccumulatorValue": "ucte.meas",
    "RemotePoint": "ucte.scada",
    "RemoteControl": "ucte.scada",
    "RemoteUnit": "ucte.scada",
    "CommunicationLink": "ucte.scada",
    "RemoteSource": "ucte.scada",
    "StateVariable": "ucte.state_variables",
    "SvVoltage": "ucte.state_variables",
    "SvShuntCompensatorSections": "ucte.state_variables",
    "SvPowerFlow": "ucte.state_variables",
    "SvStatus": "ucte.state_variables",
    "SvTapStep": "ucte.state_variables",
    "SvInjection": "ucte.state_variables",
    "EquivalentEquipment": "ucte.equivalents",
    "EquivalentShunt": "ucte.equivalents",
    "EquivalentBranch": "ucte.equivalents",
    "EquivalentNetwork": "ucte.equivalents",
    "ContingencyElement": "ucte.contingency",
    "ContingencyEquipment": "ucte.contingency",
    "Contingency": "ucte.contingency",
    "RecloseSequence": "ucte.protection",
    "ProtectionEquipment": "ucte.protection",
    "CurrentRelay": "ucte.protection",
    "SynchrocheckRelay": "ucte.protection",
    "AltTieMeas": "ucte.control_area",
    "AltGeneratingUnitMeas": "ucte.control_area",
    "ControlArea": "ucte.control_area",
    "TieFlow": "ucte.control_area",
    "ControlAreaGeneratingUnit": "ucte.control_area",
}



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" The specificatoin of limits associated with equipment and other operational entities.The specificatoin of limits associated with equipment and other operational entities.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CPSM.IEC61970.Core import IdentifiedObject
from CPSM.IEC61970.Domain import ActivePower
from CPSM.IEC61970.Domain import ApparentPower
from CPSM.IEC61970.Domain import Voltage
from CPSM.IEC61970.Domain import CurrentFlow



from enthought.traits.api import Instance, List, Property, Str
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
#  "OperationalLimit" class:
#------------------------------------------------------------------------------

class OperationalLimit(IdentifiedObject):
    """ A value associated with a specific kind of limit.A value associated with a specific kind of limit.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The limit set to which the limit values belong.The limit set to which the limit values belong.
    OperationalLimitSet = Instance("CPSM.IEC61970.OperationalLimits.OperationalLimitSet", allow_none=False,
        desc="The limit set to which the limit values belong.The limit set to which the limit values belong.",
        transient=True,
        opposite="OperationalLimitValue",
        editor=InstanceEditor(name="_operationallimitsets"))

    def _get_operationallimitsets(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.OperationalLimits.OperationalLimitSet" ]
        else:
            return []

    _operationallimitsets = Property(fget=_get_operationallimitsets)

    # Used to specify high/low and limit levels.Used to specify high/low and limit levels.
    type = Str(desc="Used to specify high/low and limit levels.Used to specify high/low and limit levels.")

    #--------------------------------------------------------------------------
    #  Begin "OperationalLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "type",
                label="Attributes"),
            VGroup("Model", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.OperationalLimits.OperationalLimit",
        title="OperationalLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OperationalLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "OperationalLimitSet" class:
#------------------------------------------------------------------------------

class OperationalLimitSet(IdentifiedObject):
    """ A set of limits associated with equipmnet.A set of limits associated with equipmnet.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The equpment to which the limit set applies.The equpment to which the limit set applies.
    Equipment = Instance("CPSM.IEC61970.Core.Equipment", allow_none=False,
        desc="The equpment to which the limit set applies.The equpment to which the limit set applies.",
        transient=True,
        opposite="OperationalLimitSet",
        editor=InstanceEditor(name="_equipments"))

    def _get_equipments(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.Core.Equipment" ]
        else:
            return []

    _equipments = Property(fget=_get_equipments)

    # Values of equipment limits.Values of equipment limits.
    OperationalLimitValue = List(Instance("CPSM.IEC61970.OperationalLimits.OperationalLimit"),
        desc="Values of equipment limits.Values of equipment limits.")

    #--------------------------------------------------------------------------
    #  Begin "OperationalLimitSet" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "Equipment", "OperationalLimitValue",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.OperationalLimits.OperationalLimitSet",
        title="OperationalLimitSet",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OperationalLimitSet" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ActivePowerLimit" class:
#------------------------------------------------------------------------------

class ActivePowerLimit(OperationalLimit):
    """ Limit on active power flow.Limit on active power flow.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Value of active power limit.Value of active power limit.
    value = ActivePower(desc="Value of active power limit.Value of active power limit.")

    #--------------------------------------------------------------------------
    #  Begin "ActivePowerLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "type", "value",
                label="Attributes"),
            VGroup("Model", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.OperationalLimits.ActivePowerLimit",
        title="ActivePowerLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ActivePowerLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ApparentPowerLimit" class:
#------------------------------------------------------------------------------

class ApparentPowerLimit(OperationalLimit):
    """ Apparent power limit.Apparent power limit.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The apparent power limit.The apparent power limit.
    value = ApparentPower(desc="The apparent power limit.The apparent power limit.")

    #--------------------------------------------------------------------------
    #  Begin "ApparentPowerLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "type", "value",
                label="Attributes"),
            VGroup("Model", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.OperationalLimits.ApparentPowerLimit",
        title="ApparentPowerLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ApparentPowerLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "VoltageLimit" class:
#------------------------------------------------------------------------------

class VoltageLimit(OperationalLimit):
    """ Operational limit applied to voltage.Operational limit applied to voltage.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Limit on voltage. High or low limit depends on the OperatoinalLimit.limitKindLimit on voltage. High or low limit depends on the OperatoinalLimit.limitKind
    value = Voltage(desc="Limit on voltage. High or low limit depends on the OperatoinalLimit.limitKindLimit on voltage. High or low limit depends on the OperatoinalLimit.limitKind")

    #--------------------------------------------------------------------------
    #  Begin "VoltageLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "type", "value",
                label="Attributes"),
            VGroup("Model", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.OperationalLimits.VoltageLimit",
        title="VoltageLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "VoltageLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CurrentLimit" class:
#------------------------------------------------------------------------------

class CurrentLimit(OperationalLimit):
    """ OIoeratuibak kimit on current.OIoeratuibak kimit on current.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Limit on current flow.Limit on current flow.
    value = CurrentFlow(desc="Limit on current flow.Limit on current flow.")

    #--------------------------------------------------------------------------
    #  Begin "CurrentLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "type", "value",
                label="Attributes"),
            VGroup("Model", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.OperationalLimits.CurrentLimit",
        title="CurrentLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CurrentLimit" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------
